@@ -31,23 +31,33 @@ System (version for POSIX threads).
 #include "gnome-vfs-job.h"
 
 #include "gnome-vfs-async-job-map.h"
-#include "gnome-vfs-job-slave.h"
 #include "gnome-vfs-job-queue.h"
+#include "gnome-vfs-private-utils.h"
 #include <errno.h>
-#include <fcntl.h>
 #include <glib/gmessages.h>
 #include <glib/gstrfuncs.h>
+#include <glib/gi18n-lib.h>
 #include <libgnomevfs/gnome-vfs-cancellable-ops.h>
 #include <libgnomevfs/gnome-vfs-context.h>
-#include <libgnomevfs/gnome-vfs-i18n.h>
 #include <libgnomevfs/gnome-vfs-backend.h>
 #include <string.h>
 #include <unistd.h>
 
-GStaticPrivate job_private = G_STATIC_PRIVATE_INIT;
+static GStaticPrivate job_private = G_STATIC_PRIVATE_INIT;
 
 #if GNOME_VFS_JOB_DEBUG
 
+char *job_debug_types[] = {
+	"open", "open as channel",
+	"create", "create symbolic link",
+	"create as channel", "close",
+	"read", "write", "seek", "read write done",
+	"load directory", "find directory",
+	"xfer", "get file info", "set file info",
+	"module callback", "file control",
+	"**error**"
+};
+
 /* FIXME bugzilla.eazel.com 1130
  * - this is should use the correct static mutex initialization macro.
  * However glibconfig.h is broken and the supplied macro gives a warning.
@@ -67,46 +77,6 @@ static gboolean dispatch_sync_job_callback          (gpointer              data)
 static void	clear_current_job 		    (void);
 static void	set_current_job 		    (GnomeVFSJob *context);
 
-static void
-set_fl (int fd, int flags)
-{
-	int val;
-
-	val = fcntl (fd, F_GETFL, 0);
-	if (val < 0) {
-		g_warning ("fcntl() F_GETFL failed: %s", strerror (errno));
-		return;
-	}
-
-	val |= flags;
-	
-	val = fcntl (fd, F_SETFL, val);
-	if (val < 0) {
-		g_warning ("fcntl() F_SETFL failed: %s", strerror (errno));
-		return;
-	}
-}
-
-static void
-clr_fl (int fd, int flags)
-{
-	int val;
-
-	val = fcntl (fd, F_GETFL, 0);
-	if (val < 0) {
-		g_warning ("fcntl() F_GETFL failed: %s", strerror (errno));
-		return;
-	}
-
-	val &= ~flags;
-	
-	val = fcntl (fd, F_SETFL, val);
-	if (val < 0) {
-		g_warning ("fcntl() F_SETFL failed: %s", strerror (errno));
-		return;
-	}
-}
-
 /*
  *   Find out whether or not a given job should be left in
  * the job map, preserving it's open VFS handle, since we
@@ -132,6 +102,7 @@ _gnome_vfs_job_complete (GnomeVFSJob *job)
 		return FALSE;
 	case GNOME_VFS_OP_READ_WRITE_DONE:
 	case GNOME_VFS_OP_FILE_CONTROL:
+	case GNOME_VFS_OP_SEEK:
 		return FALSE;
 	
 	default:
@@ -146,14 +117,18 @@ static void
 job_oneway_notify (GnomeVFSJob *job, GnomeVFSNotifyResult *notify_result)
 {
 	if (_gnome_vfs_async_job_add_callback (job, notify_result)) {
-		JOB_DEBUG (("job %u, callback %u", GPOINTER_TO_UINT (notify_result->job_handle),
-			notify_result->callback_id));
+		JOB_DEBUG (("job %u, callback %u type '%s'",
+			    GPOINTER_TO_UINT (notify_result->job_handle),
+			    notify_result->callback_id,
+			    JOB_DEBUG_TYPE (job->op->type)));
 	
 		g_idle_add (dispatch_job_callback, notify_result);
 	} else {
-		JOB_DEBUG (("Barfing on oneway cancel %u (%d)",
+		JOB_DEBUG (("Barfing on oneway cancel %u (%d) type '%s'",
 			    GPOINTER_TO_UINT (notify_result->job_handle),
-			    job->op->type));
+			    job->op->type, JOB_DEBUG_TYPE (job->op->type)));
+		/* TODO: We can leak handle here, if an open succeded.
+		 * See bug #123472 */
 		_gnome_vfs_job_destroy_notify_result (notify_result);
 	}
 }
@@ -247,6 +222,14 @@ dispatch_write_callback (GnomeVFSNotifyResult *notify_result)
 						     notify_result->specifics.write.callback_data);
 }
 
+static void
+dispatch_seek_callback (GnomeVFSNotifyResult *notify_result)
+{
+	(* notify_result->specifics.seek.callback) (notify_result->job_handle,
+						    notify_result->specifics.seek.result,
+						    notify_result->specifics.seek.callback_data);
+}
+
 static void
 dispatch_load_directory_callback (GnomeVFSNotifyResult *notify_result)
 {
@@ -383,6 +366,7 @@ _gnome_vfs_job_destroy_notify_result (GnomeVFSNotifyResult *notify_result)
 	case GNOME_VFS_OP_CREATE_AS_CHANNEL:
 	case GNOME_VFS_OP_CREATE_SYMBOLIC_LINK:
 	case GNOME_VFS_OP_WRITE:
+	case GNOME_VFS_OP_SEEK:
 	case GNOME_VFS_OP_OPEN:
 	case GNOME_VFS_OP_OPEN_AS_CHANNEL:
 	case GNOME_VFS_OP_READ:
@@ -498,7 +482,8 @@ dispatch_job_callback (gpointer data)
 	
 	notify_result = (GnomeVFSNotifyResult *) data;
 
-	JOB_DEBUG (("%u", GPOINTER_TO_UINT (notify_result->job_handle)));	
+	JOB_DEBUG (("%u type '%s'", GPOINTER_TO_UINT (notify_result->job_handle),
+		    JOB_DEBUG_TYPE (notify_result->type)));
 	
 	_gnome_vfs_async_job_callback_valid (notify_result->callback_id, &valid, &cancelled);
 	_gnome_vfs_async_job_remove_callback (notify_result->callback_id);
@@ -590,6 +575,9 @@ dispatch_job_callback (gpointer data)
 	case GNOME_VFS_OP_WRITE:
 		dispatch_write_callback (notify_result);
 		break;
+	case GNOME_VFS_OP_SEEK:
+		dispatch_seek_callback (notify_result);
+		break;
 	case GNOME_VFS_OP_FILE_CONTROL:
 		dispatch_file_control_callback (notify_result);
 		break;
@@ -734,6 +722,7 @@ gnome_vfs_op_destroy (GnomeVFSOp *op)
 		break;
 	case GNOME_VFS_OP_READ:
 	case GNOME_VFS_OP_WRITE:
+	case GNOME_VFS_OP_SEEK:
 	case GNOME_VFS_OP_CLOSE:
 	case GNOME_VFS_OP_READ_WRITE_DONE:
 		break;
@@ -741,7 +730,7 @@ gnome_vfs_op_destroy (GnomeVFSOp *op)
 		g_free (op->specifics.file_control.operation);
 		break;
 	default:
-		g_warning (_("Unknown op type %u"), op->type);
+		g_warning (_("Unknown operation type %u"), op->type);
 	}
 	
 	g_assert (gnome_vfs_context_get_cancellation (op->context) != NULL);
@@ -755,8 +744,9 @@ gnome_vfs_op_destroy (GnomeVFSOp *op)
 void
 _gnome_vfs_job_go (GnomeVFSJob *job)
 {
-	JOB_DEBUG (("new job %u, op %d, unlocking job lock",
-		GPOINTER_TO_UINT (job->job_handle), job->op->type));
+	JOB_DEBUG (("new job %u, op %d, type '%s' unlocking job lock",
+		    GPOINTER_TO_UINT (job->job_handle), job->op->type,
+		    JOB_DEBUG_TYPE (job->op->type)));
 
 	/* Fire up the async job thread. */
 	if (!_gnome_vfs_job_schedule (job)) {
@@ -856,7 +846,7 @@ serve_channel_read (GnomeVFSHandle *handle,
 
 					fd = g_io_channel_unix_get_fd (channel_out);
 					
-					clr_fl (fd, O_NONBLOCK);
+					_gnome_vfs_pipe_set_blocking (fd, TRUE);
 				} else {
 					if (written_bytes_in_buffer > 0) {
 						/* Need to shift the unwritten bytes
@@ -922,7 +912,7 @@ serve_channel_write (GnomeVFSHandle *handle,
 
 		io_result = g_io_channel_read_chars (channel_in, buffer, buffer_size,
 						     &bytes_read, NULL);
-
+                
 		if (io_result == G_IO_STATUS_AGAIN)
 			continue;
 		if (io_result != G_IO_STATUS_NORMAL || bytes_read == 0)
@@ -1028,8 +1018,9 @@ execute_open_as_channel (GnomeVFSJob *job)
 		job_oneway_notify (job, notify_result);
 		return;
 	}
+	
 
-	if (pipe (pipefd) < 0) {
+	if (_gnome_vfs_pipe (pipefd) < 0) {
 		g_warning (_("Cannot create pipe for open GIOChannel: %s"),
 			   g_strerror (errno));
 		notify_result->specifics.open_as_channel.result = GNOME_VFS_ERROR_INTERNAL;
@@ -1043,11 +1034,15 @@ execute_open_as_channel (GnomeVFSJob *job)
 	 * thread is blocking for some reason the slave can keep
 	 * reading data.
 	 */
-	set_fl (pipefd[1], O_NONBLOCK);
+	_gnome_vfs_pipe_set_blocking (pipefd[1], FALSE);
 	
+#ifndef G_OS_WIN32
 	channel_in = g_io_channel_unix_new (pipefd[0]);
 	channel_out = g_io_channel_unix_new (pipefd[1]);
-
+#else
+	channel_in = g_io_channel_win32_new_socket (pipefd[0]);
+	channel_out = g_io_channel_win32_new_socket (pipefd[1]);
+#endif
 	open_mode = open_as_channel_op->open_mode;
 	
 	if (open_mode & GNOME_VFS_OPEN_READ) {
@@ -1153,10 +1148,12 @@ execute_create_as_channel (GnomeVFSJob *job)
 	if (create_as_channel_op->uri == NULL) {
 		result = GNOME_VFS_ERROR_INVALID_URI;
 	} else {
-		result = gnome_vfs_open_uri_cancellable
+		result = gnome_vfs_create_uri_cancellable
 			(&handle,
 			 create_as_channel_op->uri,
 			 create_as_channel_op->open_mode,
+                         create_as_channel_op->exclusive,
+                         create_as_channel_op->perm,
 			 job->op->context);
 	}
 	
@@ -1174,7 +1171,7 @@ execute_create_as_channel (GnomeVFSJob *job)
 		return;
 	}
 
-	if (pipe (pipefd) < 0) {
+	if (_gnome_vfs_pipe (pipefd) < 0) {
 		g_warning (_("Cannot create pipe for open GIOChannel: %s"),
 			   g_strerror (errno));
 		notify_result->specifics.create_as_channel.result = GNOME_VFS_ERROR_INTERNAL;
@@ -1184,8 +1181,13 @@ execute_create_as_channel (GnomeVFSJob *job)
 		return;
 	}
 	
+#ifndef G_OS_WIN32
 	channel_in = g_io_channel_unix_new (pipefd[0]);
 	channel_out = g_io_channel_unix_new (pipefd[1]);
+#else
+	channel_in = g_io_channel_win32_new_socket (pipefd[0]);
+	channel_out = g_io_channel_win32_new_socket (pipefd[1]);
+#endif
 
 	notify_result->specifics.create_as_channel.channel = channel_out;
 
@@ -1267,6 +1269,30 @@ execute_write (GnomeVFSJob *job)
 	job_oneway_notify (job, notify_result);
 }
 
+static void
+execute_seek (GnomeVFSJob *job)
+{
+	GnomeVFSResult result;
+	GnomeVFSSeekOp *seek_op;
+	GnomeVFSNotifyResult *notify_result;
+
+	seek_op = &job->op->specifics.seek;
+
+	result = gnome_vfs_seek_cancellable (job->handle,
+					     seek_op->whence,
+					     seek_op->offset,
+					     job->op->context);
+
+	notify_result = g_new0 (GnomeVFSNotifyResult, 1);
+	notify_result->job_handle = job->job_handle;
+	notify_result->type = job->op->type;
+	notify_result->specifics.seek.result = result;
+	notify_result->specifics.seek.callback = (GnomeVFSAsyncSeekCallback) job->op->callback;
+	notify_result->specifics.seek.callback_data = job->op->callback_data;
+
+	job_oneway_notify (job, notify_result);
+}
+
 static void
 execute_get_file_info (GnomeVFSJob *job)
 {
@@ -1619,7 +1645,8 @@ _gnome_vfs_job_execute (GnomeVFSJob *job)
 	if (!job->cancelled) {
 		set_current_job (job);
 
-		JOB_DEBUG (("executing %u %d", id, job->op->type));
+		JOB_DEBUG (("executing %u %d type %s", id, job->op->type,
+			    JOB_DEBUG_TYPE (job->op->type)));
 
 		switch (job->op->type) {
 		case GNOME_VFS_OP_OPEN:
@@ -1646,6 +1673,9 @@ _gnome_vfs_job_execute (GnomeVFSJob *job)
 		case GNOME_VFS_OP_WRITE:
 			execute_write (job);
 			break;
+		case GNOME_VFS_OP_SEEK:
+			execute_seek (job);
+			break;
 		case GNOME_VFS_OP_LOAD_DIRECTORY:
 			execute_load_directory (job);
 			break;
